/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package sim.data;

import java.awt.Rectangle;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import sim.data.finals.Anlage;
import sim.data.finals.BadLampe;
import sim.data.finals.Computer;
import sim.data.finals.Fernseher;
import sim.data.finals.Geschirr;
import sim.data.finals.Kaffee;
import sim.data.finals.Lavalampe;
import sim.data.finals.Mikrowelle;
import sim.data.finals.Mixer;
import sim.data.finals.Ofen;
import sim.data.finals.SchlafLampe;
import sim.data.finals.SchlafSchalter;
import sim.data.finals.Telefon;
import sim.data.finals.Wanne;
import sim.data.finals.Waschmaschine;
import sim.data.finals.BadSchalter;
import sim.data.finals.FernseherNoGame;
import sim.data.finals.GameLampe;
import sim.data.finals.GameSchalter;
import sim.data.finals.KuecheLampe;
import sim.data.finals.KuecheSchalter;
import sim.data.finals.PlayStation;
import sim.data.finals.WaschLampe;
import sim.data.finals.WaschSchalter;
import sim.data.finals.WohnLampe;
import sim.data.finals.WohnSchalter;

/**
 *
 * @author Dromliuss
 */
public class Haus extends UnicastRemoteObject implements HausRemote,SimpleHouse {

    private final Room[] rooms;
    private final Gerät[] hausGeräte;
    private static int idCounter = 0;
    private final int id;
    private boolean blackOut;

    private final String owner;

    public Haus(String owner) throws RemoteException {
        id = idCounter++;
        rooms = new Room[]{new Room(new Rectangle(46, 130, 222, 227)),
            new Room(new Rectangle(276, 130, 156, 227)),
            new Room(new Rectangle(440, 130, 292, 227)),
            new Room(new Rectangle(46, 365, 261, 227)),
            new Room(new Rectangle(315, 365, 417, 227)),
            new Room(new Rectangle(740, 365, 225, 227))};
//
        SchlafLampe schlafLampe = new SchlafLampe(rooms[0]);
        BadLampe badLampe = new BadLampe(rooms[1]);
        GameLampe gameLampe = new GameLampe(rooms[2]);
        KuecheLampe kuecheLampe = new KuecheLampe(rooms[3]);
        WohnLampe wohnLampe = new WohnLampe(rooms[4]);
        WaschLampe waschLampe = new WaschLampe(rooms[5]);
        PlayStation playStation = new PlayStation();
        this.hausGeräte = new Gerät[]{new Fernseher(), new Computer(),
            new Lavalampe(), new Wanne(), new Anlage(), new Geschirr(),
            new Telefon(), new Mixer(), new Kaffee(), new Mikrowelle(),
            new Ofen(), new Waschmaschine(),
            schlafLampe, new SchlafSchalter(schlafLampe),
            badLampe, new BadSchalter(badLampe),
            gameLampe, new GameSchalter(gameLampe),
            kuecheLampe, new KuecheSchalter(kuecheLampe),
            wohnLampe, new WohnSchalter(wohnLampe),
            waschLampe, new WaschSchalter(waschLampe),
            playStation,new FernseherNoGame(playStation)};
        this.owner = owner;
        for(Gerät g:hausGeräte){
            g.setHome(this);
        }
    }

    @Override
    public int getID() throws RemoteException {
        return id;
    }

    @Override
    public String getOwner() {
        return owner;
    }

    @Override
    public GerätRemote getHausGerät(int indx) {
        return hausGeräte[indx];
    }

    public Gerät[] getHausGeräte() {
        return hausGeräte;
    }

    public void setBlackOut(boolean blackOut) {
        this.blackOut = blackOut;
    }

    @Override
    public boolean isBlackOut() {
        return blackOut;
    }




    public int getNeededpower() {
        //
        int needed = 50;
        for (Gerät g : hausGeräte) {
            if (g.isOn()) {
                needed += g.getSTROMVERBRAUCH();
            }
        }
        return needed;
    }

    @Override
    public int getHausGerätAnzahl() throws RemoteException {
        return hausGeräte.length;
    }

    @Override
    public RoomRemote getRoom(int indx) throws RemoteException {
        return rooms[indx];
    }

    @Override
    public int getRoomAnzahl() throws RemoteException {
        return rooms.length;
    }
    
    private boolean isSmart;
    public void setIsSmart(boolean isSmart) {
        this.isSmart = isSmart;
    }

    @Override
    public boolean isIsSmart() {
        return isSmart;
    }

    public void notifySmartDevices() {
        for(Gerät g:getHausGeräte()){
                g.notifySmart();
        }
    }
}
