package sim.data;

import java.util.ArrayList;
import java.util.HashMap;
import sim.graphics.InnerConsole;

/**
 *
 * @author Dromliuss
 */
public class PowerPlant {

    private static PowerPlant powerPlant;

    public static PowerPlant get() {
        if (powerPlant != null) {
            return powerPlant;
        }
        throw new RuntimeException("Powerplant not yet created!!!");
    }

    public static void createPowerplant(Neighborhood neighborhood) {
        if (powerPlant == null) {
            powerPlant = new PowerPlant(neighborhood);
            return;
        }
        throw new RuntimeException("Powerplant already created!!!");
    }

    public double wasted;
    public double blackouttime = 0;
    private Mode mode = Mode.Manual;
    public static int maxPower = 10000;
    private final Neighborhood neighborhood;
    public boolean blackout = false;
    private static final int rounding = 100;
    private final HashMap<PowerPlantType, Integer> powerPlants = new HashMap<>();
    HashMap<Mode, PowerControler> controller = new HashMap<>();

    private PowerPlant(Neighborhood neighborhood) {
        this.neighborhood = neighborhood;
        powerPlants.put(PowerPlantType.Coal, 1);
        powerPlants.put(PowerPlantType.Solar, 1);
        powerPlants.put(PowerPlantType.Nuclear, 0);
        controller.put(Mode.Manual, null);
        controller.put(Mode.Automatic, new PowerControler() {

            @Override
            public void action() {
                if (getNetzFrequenz() < 50) {
                    manipulatePower(PowerPlantType.Coal, getPowerPlants().get(PowerPlantType.Coal) + 1);
                }
                if (getNetzFrequenz() > 50.3) {
                    manipulatePower(PowerPlantType.Coal, getPowerPlants().get(PowerPlantType.Coal) - 1);
                }
            }
        });
        controller.put(Mode.Smart, new PowerControler() {
            private final int schwelle = 1000;

            @Override
            public void action() {
                if (getNetzFrequenz() < 50) {
                    manipulatePower(PowerPlantType.Coal, getPowerPlants().get(PowerPlantType.Coal) + 1);
                }

                if (getNetzFrequenz() > 50) {
                    for (Haus house : PowerPlant.this.neighborhood.getNeighborhood()) {
                        if (getNetzFrequenz() < 50) {
                            break;
                        }
                      //  System.out.println("NOTIFY!");
                        house.notifySmartDevices();
                    }
                }
                if (getNetzFrequenz() > 50.3) {
                    manipulatePower(PowerPlantType.Coal, getPowerPlants().get(PowerPlantType.Coal) - 1);
                }
            }
        });
    }
    private long lastControllertiming = System.currentTimeMillis();
    private long lastwastedtiming = System.currentTimeMillis();
    private long blackOutStart = 0;

    public void run() {
        maxPower = Math.max(neighborhood.getNeighborhood().size() * 3600, 2000);
        boolean prevBlack = blackout;
        blackout = getNetzFrequenz() < 49 || getNetzFrequenz() > 51;
        if (!prevBlack && blackout) {
            blackOutStart = System.currentTimeMillis();
        }
        if (prevBlack && !blackout) {
            blackouttime += (double) (System.currentTimeMillis() - blackOutStart) / World.millSecPerHour;
        }
        for (Haus h : neighborhood.getNeighborhood()) {
            h.setBlackOut(blackout);
            h.setIsSmart(mode == Mode.Smart);
        }
        if (System.currentTimeMillis() - lastControllertiming > 1500) {
            if (controller.get(mode) != null) {
                controller.get(mode).action();
            }
            lastControllertiming = System.currentTimeMillis();
        }
        if (System.currentTimeMillis() - lastwastedtiming > 500) {
            wasted += Math.max(0, (double) (getGeneratedPower() - getrealGesamtStromverbrauch()) * (500d / (double) World.millSecPerHour)) * getPrice();
            lastwastedtiming = System.currentTimeMillis();
        }
        // System.out.println("blackouttime: " + (int) blackouttime + ":" + (int) ((blackouttime - (int) blackouttime) * 60));
    }

    public void stopWatch() {
        if (blackout) {
            blackouttime += (double) (System.currentTimeMillis() - blackOutStart) / World.millSecPerHour;
            //  System.out.println("blackouttime: " + (int) blackouttime + ":" + (int) ((blackouttime - (int) blackouttime) * 60));
        }
    }

    public double getWasted() {
        return wasted;
    }

    public void manipulatePower(PowerPlantType plantType, int value) {
        if (value < 0) {
            return;
        }
        if (Math.abs(value - powerPlants.get(plantType)) == 1) {
            String txt = "Ein " + plantType.getKraftwerkName() + " wurde ";
            if (value - powerPlants.get(plantType) == 1) {
                txt += "hochgefahren.";
            } else {
                txt += "runtergefahren.";
            }
            InnerConsole.get().ausgeben(txt);
        } else {
            System.out.println("whyyyyy??");
        }
        powerPlants.put(plantType, Math.max(0, value));
    }

    public HashMap<PowerPlantType, Integer> getPowerPlants() {
        return powerPlants;
    }

    public double getPrice() {
        return 0.2 * powerPlants.get(PowerPlantType.Coal) + 0.152 * powerPlants.get(PowerPlantType.Solar) + 0.5 * powerPlants.get(PowerPlantType.Nuclear);
    }

    public int getGeneratedPower() {
        return 200 * powerPlants.get(PowerPlantType.Coal) + (int) (100 * (-Math.cos(World.getWorld().getTime() * Math.PI / 12) + 2) * (double) powerPlants.get(PowerPlantType.Solar)) + 1000 * powerPlants.get(PowerPlantType.Nuclear);
    }

    public int getGesamtStromverbrauch() {
        return mode == Mode.Smart || true ? getrealGesamtStromverbrauch() : (getrealGesamtStromverbrauch() / rounding + 1) * rounding;
    }

    private int getrealGesamtStromverbrauch() {
        int neededPower = 0;
        for (Haus h : neighborhood.getNeighborhood()) {
            //
            neededPower += 50;
            for (Gerät g : h.getHausGeräte()) {
                if (g.isOn()) {
                    neededPower += g.getSTROMVERBRAUCH();
                }
            }
        }
        return neededPower;
    }

    public Neighborhood getNeighborhood() {
        return neighborhood;
    }

    public void setMode(Mode mode) {
        this.mode = mode;
    }

    public Mode getMode() {
        return mode;
    }

    public void smartNotify(Gerät g) {
        if (mode == Mode.Smart) {
            SmartInfo smartInfo = new SmartInfo(g);
            InnerConsole.get().verabeiteInformationen(smartInfo);
            report.add(smartInfo.DATUM+": "+smartInfo.HAUSHALT+" schaltete "+smartInfo.GERÄT+((smartInfo.istEINGESCHALTET)?" ein.":" aus."));
        }
    }
    public ArrayList<String> report = new ArrayList<>();

    public float getNetzFrequenz() {
        int diff = (getGeneratedPower() - getrealGesamtStromverbrauch());
        if (diff < 400) {
            return 1f / 400 * diff + 49;
        }
        if (diff < 800) {
            return 50;
        }
        return (diff + 34200) / 700f;
    }
}
