/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sim.graphics;

import com.sun.java.accessibility.util.AWTEventMonitor;
import help.math.IntPosition;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ContainerAdapter;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.EnumMap;
import java.util.HashMap;
import java.util.List;
import javax.imageio.ImageIO;
import javax.swing.JTextArea;
import sim.data.Mode;
import sim.data.PowerPlant;
import sim.data.PowerPlantType;
import sim.data.World;

/**
 *
 * @author Dromlius
 */
public class PowerplantPanel extends DoubleBufferedPanel implements ScreenNode {

    private final PowerplantPanel selfReference = this;
    private final IntPosition scrollPoint = new IntPosition();
    private final int scrollSpeed = 1;
    private static final Font font = new Font("Arial", Font.BOLD, 24);
     private static final Font consoleFont = new Font("Power Red and Green", Font.BOLD, 16);
    private static final Color color = new Color(200, 200, 200, 200);
    private BufferedImage up;
    private BufferedImage down;
    //
    //   private final int RESOLUTION;
    //
    //
    private BufferedImage consoleBG;
    private BufferedImage mainBg;
    private BufferedImage danger;
    private boolean shift;
    //
    @SuppressWarnings({"CallToThreadStartDuringObjectConstruction", "LeakingThisInConstructor"})
    public PowerplantPanel(int width, int height, final boolean game) {
        super(width, height);
        final PowerPlant powerPlant = PowerPlant.get();
        actionAreas = new EnumMap<>(Trigger.class);
        try {
            consoleBG = ImageIO.read(new File("./cache/consoleBG.png").toURI().toURL());
            mainBg = ImageIO.read(new File("./cache/serverBg.png").toURI().toURL());
            danger = ImageIO.read(new File("./cache/danger.png").toURI().toURL());
            up = ImageIO.read(new File("./cache/up.png").toURI().toURL());
            down = ImageIO.read(new File("./cache/down.png").toURI().toURL());
        } catch (IOException ioe) {
            System.out.println("IMG ;( 67543");
        }
        addKeyListener(new KeyAdapter() {

            @Override
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_S){
                    shift=true;
                }
            }

            @Override
            public void keyReleased(KeyEvent e) {
                if(e.getKeyCode() == KeyEvent.VK_S){
                    shift=false;
                }
            }
           
            
        });
        //-----------
        children = new ArrayList<>();
        //
        ScreenArea modeSelector = new ScreenArea("ModeSelector", 0, 0, 200, 70, this) {

            @Override
            public DrawCondition drawCondition(DrawCondition parentalDrawCondition) {
                return parentalDrawCondition;
            }
            boolean set = false;

            @Override
            public void internalDraw() {
                if (!set) {
                    addActionArea(Trigger.MouseClick, this, new ScreenFixActionArea(new Rectangle(0 * getScreenAreaWidth() / 3, 0, 1 * getScreenAreaWidth() / 3, getScreenAreaHeight()), new Runnable() {

                        @Override
                        public void run() {
                            powerPlant.setMode(Mode.Manual);
                        }
                    }));
                    addActionArea(Trigger.MouseClick, this, new ScreenFixActionArea(new Rectangle(1 * getScreenAreaWidth() / 3, 0, 1 * getScreenAreaWidth() / 3, getScreenAreaHeight()), new Runnable() {

                        @Override
                        public void run() {
                            if (!game) {
                                powerPlant.setMode(Mode.Automatic);
                            }
                        }
                    }));
                    addActionArea(Trigger.MouseClick, this, new ScreenFixActionArea(new Rectangle(2 * getScreenAreaWidth() / 3, 0, 1 * getScreenAreaWidth() / 3, getScreenAreaHeight()), new Runnable() {

                        @Override
                        public void run() {
                            if (!game && shift) {
                                powerPlant.setMode(Mode.Smart);
                            }
                        }
                    }));
                    set = true;
                }
                Graphics2D screenAreaGraphics = getScreenAreaGraphics();
                if (powerPlant.getMode() == Mode.Manual) {
                    screenAreaGraphics.setColor(Color.GREEN);
                } else {
                    screenAreaGraphics.setColor(Color.GRAY);
                }
                screenAreaGraphics.fillRect(0 * getScreenAreaWidth() / 3, 0, 1 * getScreenAreaWidth() / 3, getScreenAreaHeight());
                screenAreaGraphics.setColor(Color.BLACK);
                screenAreaGraphics.drawString("Manual", 10 + 0 * getScreenAreaWidth() / 3, 40);
                if (powerPlant.getMode() == Mode.Automatic) {
                    screenAreaGraphics.setColor(Color.GREEN);
                } else {
                    screenAreaGraphics.setColor(Color.GRAY.darker());
                }
                screenAreaGraphics.fillRect(1 * getScreenAreaWidth() / 3, 0, 1 * getScreenAreaWidth() / 3, getScreenAreaHeight());
                screenAreaGraphics.setColor(Color.BLACK);
                screenAreaGraphics.drawString("Automatik", 10 + 1 * getScreenAreaWidth() / 3, 40);
                if (powerPlant.getMode() == Mode.Smart) {
                    screenAreaGraphics.setColor(Color.GREEN);
                } else {
                    screenAreaGraphics.setColor(Color.GRAY.darker().darker());
                }
                screenAreaGraphics.fillRect(2 * getScreenAreaWidth() / 3, 0, 1 * getScreenAreaWidth() / 3 + 1, getScreenAreaHeight());
                screenAreaGraphics.setColor(Color.BLACK);
                screenAreaGraphics.drawString("Smart", 10 + 2 * getScreenAreaWidth() / 3, 40);

            }
        };
        ScreenArea powerbar = new ScreenArea("PowerBar", 0, new RightBelowOrientation(modeSelector), modeSelector.getScreenAreaWidth(), 600, this) {
            @Override
            public DrawCondition drawCondition(DrawCondition parentalDrawCondition) {
                return parentalDrawCondition;
            }

            @Override
            public void internalDraw() {
                getScreenAreaGraphics().setColor(new Color(0, 0, 0, 200));
                getScreenAreaGraphics().fillRect(0, 0, getScreenAreaWidth(), getScreenAreaHeight());
                getScreenAreaGraphics().setColor(Color.YELLOW);
                getScreenAreaGraphics().fillRect(0, getScreenAreaHeight() - (int) (getScreenAreaHeight() * ((double) powerPlant.getGeneratedPower() / PowerPlant.maxPower)), getScreenAreaWidth(), (int) (getScreenAreaHeight() * ((double) powerPlant.getGeneratedPower() / PowerPlant.maxPower)));
                if (powerPlant.getMode() == Mode.Smart) {
                    int oldHeigth = 0;
                    for (int i = 0; i < powerPlant.getNeighborhood().getNeighborhood().size(); i++) {
                        Color c = Color.RED;
                        if (i % 2 == 0) {
                            getScreenAreaGraphics().setColor(c.brighter());
                        } else {
                            getScreenAreaGraphics().setColor(c.darker());
                        }
                        getScreenAreaGraphics().fillRect(0, getScreenAreaHeight() - oldHeigth - (int) (getScreenAreaHeight() * ((double) powerPlant.getNeighborhood().getNeighborhood().get(i).getNeededpower() / PowerPlant.maxPower)), 2 * getScreenAreaWidth() / 3, (int) (getScreenAreaHeight() * ((double) powerPlant.getNeighborhood().getNeighborhood().get(i).getNeededpower() / PowerPlant.maxPower)));
                        getScreenAreaGraphics().setColor(Color.BLACK);
                        if (powerPlant.getNeighborhood().getNeighborhood().get(i).getNeededpower() > 100) {
                            getScreenAreaGraphics().drawString(powerPlant.getNeighborhood().getNeighborhood().get(i).getOwner() + ": " + powerPlant.getNeighborhood().getNeighborhood().get(i).getNeededpower() + "W", 10, getScreenAreaHeight() - oldHeigth - ((int) (getScreenAreaHeight() * ((double) powerPlant.getNeighborhood().getNeighborhood().get(i).getNeededpower() / PowerPlant.maxPower))) / 2 + 5);
                        }
                        oldHeigth += (int) (getScreenAreaHeight() * ((double) powerPlant.getNeighborhood().getNeighborhood().get(i).getNeededpower() / PowerPlant.maxPower));
                    }
                } else {
                    getScreenAreaGraphics().setColor(Color.RED);
                    getScreenAreaGraphics().fillRect(0, getScreenAreaHeight() - (((int) (getScreenAreaHeight() * ((double) powerPlant.getGesamtStromverbrauch() / PowerPlant.maxPower)))),
                            2 * getScreenAreaWidth() / 3, ((int) (getScreenAreaHeight() * ((double) powerPlant.getGesamtStromverbrauch() / PowerPlant.maxPower))));
                }

                if (powerPlant.blackout) {
                    if ((System.currentTimeMillis() / 500) % 3 < 2) {
                        getScreenAreaGraphics().drawImage(danger, (getScreenAreaWidth() - danger.getWidth()) / 2, (getScreenAreaHeight() - danger.getHeight()) / 2, null);
                    }
                }
            }
        };
        //   System.out.println(powerbar);
        ScreenArea manageScreen = new ScreenArea("ManageScreen", new RightBelowOrientation(powerbar), 0, new RightBelowOrientation(powerbar), powerbar.getScreenAreaHeight() - 100, this) {
            private BufferedImage bg;

            @Override
            public DrawCondition drawCondition(DrawCondition parentalDrawCondition) {
                return parentalDrawCondition;
            }

            @Override
            public void internalDraw() {
              /*  if (bg == null) {
                    try {
                        bg = ImageIO.read(new File("./cache/simpsons.PNG").toURI().toURL());

                    } catch (IOException ioe) {
                        System.out.println("cant load powerPlantImg");
                    }
                }*/
                Graphics mainGraphics = getScreenAreaGraphics();
                mainGraphics.setColor(Color.red);
                //  mainGraphics.fillRect(0, 0, 9999, 9999);
              //  mainGraphics.drawImage(bg, 0, 0, null);
            }
        };
        ScreenArea coalScreen = new ScreenArea("CoalScreen", 0, 0, 1f / 3f, 1f, manageScreen) {
            private BufferedImage bg;

            @Override
            public DrawCondition drawCondition(DrawCondition parentalDrawCondition) {
                return parentalDrawCondition;
            }
            boolean set = false;

            @Override
            public void internalDraw() {
                if (!set) {
                    try {
                        getScreenAreaGraphics().setFont(font);
                        bg = ImageIO.read(new File("./cache/coal.png").toURI().toURL());
                    } catch (IOException ioe) {
                        System.out.println("cant load Coal!!");
                    }
                    addActionArea(Trigger.MouseClick, this, new ScreenFixActionArea(new Rectangle(getScreenAreaWidth() / 2 - 15, 120, 30, 30), new Runnable() {

                        @Override
                        public void run() {
                            powerPlant.manipulatePower(PowerPlantType.Coal, powerPlant.getPowerPlants().get(PowerPlantType.Coal) + 1);
                        }
                    }));
                    addActionArea(Trigger.MouseClick, this, new ScreenFixActionArea(new Rectangle(getScreenAreaWidth() / 2 - 15, 200, 30, 30), new Runnable() {

                        @Override
                        public void run() {
                            if (powerPlant.getPowerPlants().get(PowerPlantType.Coal) > 0) {
                                powerPlant.manipulatePower(PowerPlantType.Coal, powerPlant.getPowerPlants().get(PowerPlantType.Coal) - 1);
                            }
                        }
                    }));
                    set = true;
                    JTextArea txt = new JTextArea(" Kohlekraftwerke bieten Flexibilität\n in der Stromerzeugung, sind dafür\n jedoch teuer.");
                    txt.setVisible(true);
                    txt.setEditable(false);
                    txt.setForeground(Color.BLACK);
                    txt.setHighlighter(null);
                    txt.setBackground(color);
                    txt.setBorder(javax.swing.BorderFactory.createEmptyBorder());
                    txt.setFont(font.deriveFont(12f));
                    txt.setBounds(getAbsoulteX(), getAbsoulteY() + getScreenAreaHeight() - 100, getScreenAreaWidth(), 100);
                    add(txt);
                }
                Graphics2D screenAreaGraphics = getScreenAreaGraphics();
                screenAreaGraphics.drawImage(bg, 0, 0, null);
                screenAreaGraphics.setColor(Color.BLACK);
                screenAreaGraphics.drawString("Kohlekraftwerke", (getScreenAreaWidth() - screenAreaGraphics.getFontMetrics().stringWidth("Kohlekraftwerke")) / 2, 50);
                screenAreaGraphics.setColor(color);
                screenAreaGraphics.fillRect(getScreenAreaWidth() / 2 - 15, 120, 30, 30);
                screenAreaGraphics.drawImage(up, getScreenAreaWidth() / 2 - 15, 120, null);
                screenAreaGraphics.setColor(color);
                screenAreaGraphics.fillRect(getScreenAreaWidth() / 2 - 15, 160, 30, 30);
                screenAreaGraphics.setColor(Color.BLACK);
                screenAreaGraphics.drawString(powerPlant.getPowerPlants().get(PowerPlantType.Coal) + "", (getScreenAreaWidth() - screenAreaGraphics.getFontMetrics().stringWidth(powerPlant.getPowerPlants().get(PowerPlantType.Coal) + "")) / 2, 160 + 20);
                if (powerPlant.getPowerPlants().get(PowerPlantType.Coal) > 0) {
                    screenAreaGraphics.setColor(color);
                    screenAreaGraphics.fillRect(getScreenAreaWidth() / 2 - 15, 200, 30, 30);
                    screenAreaGraphics.drawImage(down, getScreenAreaWidth() / 2 - 15, 200, null);
                }
            }
        };
        ScreenArea solarScreen = new ScreenArea("SolarScreen", new RightBelowOrientation(coalScreen), 0, 1f / 3f, 1f, manageScreen) {
            private BufferedImage bg;

            @Override
            public DrawCondition drawCondition(DrawCondition parentalDrawCondition) {
                return parentalDrawCondition;
            }
            boolean set = false;

            private int manipulate = 0;
            private double timeStart;
            private final double duration = 3000;

            @Override
            public void internalDraw() {
                if (!set) {
                    try {
                        getScreenAreaGraphics().setFont(font);
                        bg = ImageIO.read(new File("./cache/solar.png").toURI().toURL());
                    } catch (IOException ioe) {
                        System.out.println("cant load Solar!!");
                    }
                    addActionArea(Trigger.MouseClick, this, new ScreenFixActionArea(new Rectangle(getScreenAreaWidth() / 2 - 15, 120, 30, 30), new Runnable() {

                        @Override
                        public void run() {
                            if (manipulate == 0) {
                                manipulate = 1;
                                timeStart = System.currentTimeMillis();
                            }
                        }
                    }));
                    addActionArea(Trigger.MouseClick, this, new ScreenFixActionArea(new Rectangle(getScreenAreaWidth() / 2 - 15, 200, 30, 30), new Runnable() {

                        @Override
                        public void run() {
                            if (manipulate == 0 && powerPlant.getPowerPlants().get(PowerPlantType.Solar) > 0) {
                                manipulate = -1;
                                timeStart = System.currentTimeMillis();
                            }
                        }
                    }));
                    set = true;
                    JTextArea txt = new JTextArea(" Solaranlagen erzeugen am billigsten\n Strom. Ihre Effizienz hängt jedoch\n stark von der Tageszeit ab.");
                    txt.setVisible(true);
                    txt.setEditable(false);
                    txt.setForeground(Color.BLACK);
                    txt.setHighlighter(null);
                    txt.setBackground(color);
                    txt.setBorder(javax.swing.BorderFactory.createEmptyBorder());
                    txt.setFont(font.deriveFont(11.5f));
                    txt.setBounds(getAbsoulteX(), getAbsoulteY() + getScreenAreaHeight() - 100, getScreenAreaWidth(), 100);
                    add(txt);
                }
                Graphics2D screenAreaGraphics = getScreenAreaGraphics();
                screenAreaGraphics.drawImage(bg, 0, 0, null);
                screenAreaGraphics.setColor(Color.BLACK);
                screenAreaGraphics.drawString("Solarkraftwerke", (getScreenAreaWidth() - screenAreaGraphics.getFontMetrics().stringWidth("Solarkraftwerke")) / 2, 50);
                if (manipulate == 0) {
                    screenAreaGraphics.setColor(color);
                    screenAreaGraphics.fillRect(getScreenAreaWidth() / 2 - 15, 120, 30, 30);
                    screenAreaGraphics.drawImage(up, getScreenAreaWidth() / 2 - 15, 120, null);
                    if (powerPlant.getPowerPlants().get(PowerPlantType.Solar) > 0) {
                        screenAreaGraphics.setColor(color);
                        screenAreaGraphics.fillRect(getScreenAreaWidth() / 2 - 15, 200, 30, 30);
                        screenAreaGraphics.drawImage(down, getScreenAreaWidth() / 2 - 15, 200, null);
                    }
                } else {
                    screenAreaGraphics.setColor(Color.RED);
                    screenAreaGraphics.fillRect(getScreenAreaWidth() / 2 - 15, 190, (int) (30 * ((System.currentTimeMillis() - timeStart) / duration)), 5);
                    if (System.currentTimeMillis() - timeStart > duration) {
                        powerPlant.manipulatePower(PowerPlantType.Solar, powerPlant.getPowerPlants().get(PowerPlantType.Solar) + manipulate);
                        manipulate = 0;
                    }
                }

                screenAreaGraphics.setColor(color);
                screenAreaGraphics.fillRect(getScreenAreaWidth() / 2 - 15, 160, 30, 30);

                screenAreaGraphics.setColor(Color.BLACK);
                screenAreaGraphics.drawString(powerPlant.getPowerPlants().get(PowerPlantType.Solar) + "", (getScreenAreaWidth() - screenAreaGraphics.getFontMetrics().stringWidth(powerPlant.getPowerPlants().get(PowerPlantType.Solar) + "")) / 2, 160 + 20);

            }
        };
        ScreenArea nuclearScreen = new ScreenArea("NuclearScreen", new RightBelowOrientation(solarScreen), 0, new RightBelowOrientation(solarScreen), 1f, manageScreen) {
            private BufferedImage bg;

            @Override
            public DrawCondition drawCondition(DrawCondition parentalDrawCondition) {
                return parentalDrawCondition;
            }
            boolean set = false;

            private int manipulate = 0;
            private double timeStart;
            private final double duration = 5000;

            @Override
            public void internalDraw() {
                if (!set) {
                    try {
                        getScreenAreaGraphics().setFont(font);
                        bg = ImageIO.read(new File("./cache/nuclear.png").toURI().toURL());
                    } catch (IOException ioe) {
                        System.out.println("cant load Atomic!!");
                    }
                    addActionArea(Trigger.MouseClick, this, new ScreenFixActionArea(new Rectangle(getScreenAreaWidth() / 2 - 15, 120, 30, 30), new Runnable() {

                        @Override
                        public void run() {
                            if (manipulate == 0) {
                                manipulate = 1;
                                timeStart = System.currentTimeMillis();
                            }
                        }
                    }));
                    addActionArea(Trigger.MouseClick, this, new ScreenFixActionArea(new Rectangle(getScreenAreaWidth() / 2 - 15, 200, 30, 30), new Runnable() {

                        @Override
                        public void run() {
                            if (manipulate == 0 && powerPlant.getPowerPlants().get(PowerPlantType.Nuclear) > 0) {
                                manipulate = -1;
                                timeStart = System.currentTimeMillis();
                            }
                        }
                    }));
                    set = true;
                    JTextArea txt = new JTextArea(" Atomkraftwerke sind ein billiger\n Weg Strom zu generieren. Sie sind\n jedoch schwer zu regulieren und\n produzieren dauerhaft eine große\n Menge Elektrizität.");
                    txt.setVisible(true);
                    txt.setEditable(false);
                    txt.setForeground(Color.BLACK);
                    txt.setHighlighter(null);
                    txt.setBackground(color);
                    txt.setBorder(javax.swing.BorderFactory.createEmptyBorder());
                    txt.setFont(font.deriveFont(12f));
                    txt.setBounds(getAbsoulteX(), getAbsoulteY() + getScreenAreaHeight() - 100, getScreenAreaWidth(), 100);
                    add(txt);
                }
                Graphics2D screenAreaGraphics = getScreenAreaGraphics();
                screenAreaGraphics.drawImage(bg, 0, 0, null);
                screenAreaGraphics.setColor(Color.BLACK);
                screenAreaGraphics.drawString(
                        "Atomkraftwerke", (getScreenAreaWidth() - screenAreaGraphics.getFontMetrics().stringWidth("Atomkraftwerke")) / 2, 50);
                if (manipulate == 0) {
                    screenAreaGraphics.setColor(color);
                    screenAreaGraphics.fillRect(getScreenAreaWidth() / 2 - 15, 120, 30, 30);
                    screenAreaGraphics.drawImage(up, getScreenAreaWidth() / 2 - 15, 120, null);
                    if (powerPlant.getPowerPlants().get(PowerPlantType.Nuclear) > 0) {
                        screenAreaGraphics.setColor(color);
                        screenAreaGraphics.fillRect(getScreenAreaWidth() / 2 - 15, 200, 30, 30);
                        screenAreaGraphics.drawImage(down, getScreenAreaWidth() / 2 - 15, 200, null);
                    }
                } else {
                    screenAreaGraphics.setColor(Color.RED);
                    screenAreaGraphics.fillRect(getScreenAreaWidth() / 2 - 15, 190, (int) (30 * ((System.currentTimeMillis() - timeStart) / duration)), 5);
                    if (System.currentTimeMillis() - timeStart > duration) {
                        powerPlant.manipulatePower(PowerPlantType.Nuclear, powerPlant.getPowerPlants().get(PowerPlantType.Nuclear) + manipulate);
                        manipulate = 0;
                    }
                }

                screenAreaGraphics.setColor(color);

                screenAreaGraphics.fillRect(getScreenAreaWidth() / 2 - 15, 160, 30, 30);

                screenAreaGraphics.setColor(Color.BLACK);

                screenAreaGraphics.drawString(powerPlant.getPowerPlants().get(PowerPlantType.Nuclear) + "", (getScreenAreaWidth() - screenAreaGraphics.getFontMetrics().stringWidth(powerPlant.getPowerPlants().get(PowerPlantType.Nuclear) + "")) / 2, 160 + 20);

            }
        };
        ScreenArea infoScreen = new ScreenArea("InfoScreen", new RightBelowOrientation(powerbar), new RightBelowOrientation(manageScreen), new RightBelowOrientation(powerbar), 200, this) {

            @Override
            public DrawCondition drawCondition(DrawCondition parentalDrawCondition) {
                return parentalDrawCondition;
            }

            @Override
            public void internalDraw() {
                Graphics2D screenAreaGraphics = getScreenAreaGraphics();
                screenAreaGraphics.setFont(font);
                screenAreaGraphics.setColor(Color.RED);
                if (powerPlant.getMode() == Mode.Smart) {
                    screenAreaGraphics.drawString("Verbrauch: " + powerPlant.getGesamtStromverbrauch() + " W", 20, 60);
                } else {
                    screenAreaGraphics.drawString("Verbrauch: ~" + powerPlant.getGesamtStromverbrauch() + " W", 20, 60);
                }
                screenAreaGraphics.setColor(Color.YELLOW);
                screenAreaGraphics.drawString("Erzeugt: " + powerPlant.getGeneratedPower() + " W", 20, 120);
                
                screenAreaGraphics.setColor(Color.WHITE);
                
                screenAreaGraphics.drawString("Netzfrequenz: " +String.format("%.2f", powerPlant.getNetzFrequenz())   + " Hz", 20, 160);
                //
                screenAreaGraphics.setColor(Color.WHITE);
                screenAreaGraphics.drawString("Kosten: " + ((int) (powerPlant.getPrice() * 100)) / 100d + " €", 380, 60);

                screenAreaGraphics.setColor(Color.WHITE);
                if ((int) ((World.getWorld().getTime() - (int) World.getWorld().getTime()) * 60) < 10) {
                    screenAreaGraphics.drawString("Zeit: " + (int) World.getWorld().getTime() + ":0" + (int) ((World.getWorld().getTime() - (int) World.getWorld().getTime()) * 60), 380, 120);
                } else {
                    screenAreaGraphics.drawString("Zeit: " + (int) World.getWorld().getTime() + ":" + (int) ((World.getWorld().getTime() - (int) World.getWorld().getTime()) * 60), 380, 120);
                }
            }

        };
        ScreenArea console = new ScreenArea("Console", 0, new RightBelowOrientation(powerbar), 1f, new RightBelowOrientation(powerbar), this) {

            @Override
            public DrawCondition drawCondition(DrawCondition parentalDrawCondition) {
                return parentalDrawCondition;
            }

            
            boolean fontSet = false;
            @Override
            public void internalDraw() {
                if(!fontSet){
                    getScreenAreaGraphics().setFont(consoleFont);
                    getScreenAreaGraphics().setColor(Color.LIGHT_GRAY);
                    fontSet = true;
                }
                for (int x = 0; x <= getScreenAreaWidth(); x += consoleBG.getWidth()) {
                    for (int y = 0; y <= getScreenAreaHeight(); y += consoleBG.getHeight()) {
                        getScreenAreaGraphics().drawImage(consoleBG, x, y, null);
                    }
                }
                int line=0;
                for(String s:InnerConsole.get().text){
                    getScreenAreaGraphics().drawString(s, 10, 20+line*30);
                    line++;
                }
            }
        };
        setFocusable(true);

        MouseAdapter mouseAdapter = new MouseAdapter() {
            @Override
            public void mouseMoved(MouseEvent e) {
                ScreenNode insideNode = insideNode(e.getPoint());
                try {
                    for (ScreenActionArea actionArea : actionAreas.get(Trigger.MouseHover).get(insideNode)) {
                        if (actionArea.getArea().contains(e.getPoint())) {
                            actionArea.getAction().run();
                        }
                    }
                } catch (NullPointerException npe) {
                }

            }

            @Override
            public void mousePressed(MouseEvent e) {
                ScreenNode insideNode = insideNode(e.getPoint());
                try {
                    for (ScreenActionArea actionArea : actionAreas.get(Trigger.MouseClick).get(insideNode)) {
                        if (actionArea.getArea().contains(e.getPoint())) {
                            actionArea.getAction().run();
                        }
                    }
                } catch (NullPointerException npe) {
                }
            }

            private ScreenNode insideNode(Point p) {

                ScreenNode recursion = selfReference;
                while (!recursion.getChildren().isEmpty()) {
                    boolean newChild = false;
                    for (int indx = 0; indx < recursion.getChildren().size() && !newChild; indx++) {
                        if (recursion.getChildren().get(indx).internalDrawCondition() == DrawCondition.HIDE) {
                            continue;
                        }
                        if (recursion.getChildren().get(indx).getAbsoluteArea().contains(p)) {
                            recursion = recursion.getChildren().get(indx);
                            newChild = true;
                        }
                    }
                    if (!newChild) {
                        return recursion;
                    }
                }
                return recursion;
            }

        };

        addMouseMotionListener(mouseAdapter);

        addMouseListener(mouseAdapter);

        addKeyListener(
                new KeyAdapter() {
                }
        );
    }
    private long startTime=System.currentTimeMillis();
    @Override
    @SuppressWarnings("CallToThreadDumpStack")
    public void internalDraw() {
        for (int x = 0; x <= getWidth() / mainBg.getWidth(); x++) {
            for (int y = 0; y <= getHeight() / mainBg.getHeight(); y++) {
                graphics.drawImage(mainBg, x * mainBg.getWidth(), y * mainBg.getHeight(), null);
            }
        }
        /*  graphics.setColor(Color.red);
         graphics.setFont(new Font("Arial", Font.BOLD, 32));
         if ((int) ((World.getWorld().getTime() - (int) World.getWorld().getTime()) * 60) < 10) {
         graphics.drawString((int) World.getWorld().getTime() + ":0" + (int) ((World.getWorld().getTime() - (int) World.getWorld().getTime()) * 60), 700, 800);
         } else {
         graphics.drawString((int) World.getWorld().getTime() + ":" + (int) ((World.getWorld().getTime() - (int) World.getWorld().getTime()) * 60), 700, 800);
         }*/
        for (ScreenNode child : getChildren()) {
            child.draw();
        }
    }

    @Override
    public int getScreenAreaWidth() {
        return getWidth();
    }

    @Override
    public int getScreenAreaHeight() {
        return getHeight();
    }
    public EnumMap<Trigger, HashMap<ScreenNode, ArrayList<ScreenActionArea>>> actionAreas;

    @Override
    public Rectangle getAbsoluteArea() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public int getAbsoulteX() {
        return 0;
    }

    @Override
    public int getAbsoulteY() {
        return 0;
    }

    private List<ScreenNode> children;

    @Override
    public DrawCondition internalDrawCondition() {
        return DrawCondition.VISIBLE;
    }

    @Override
    public void drawImage(IntPosition position, BufferedImage bufferedImage) {
        graphics.drawImage(bufferedImage, position.getX(), position.getY(), null);
    }

    @Override
    public List<ScreenNode> getChildren() {
        return children;
    }

    @Override
    public void addChild(ScreenNode child) {
        children.add(child);
    }

    @Override
    public void addActionArea(sim.graphics.Trigger trigger, sim.graphics.ScreenNode owner, sim.graphics.ScreenActionArea actionArea) {
        if (actionAreas.get(trigger) == null) {
            actionAreas.put(trigger, new HashMap<ScreenNode, ArrayList<ScreenActionArea>>());
        }
        if (actionAreas.get(trigger).get(owner) == null) {
            actionAreas.get(trigger).put(owner, new ArrayList<ScreenActionArea>());
        }
        actionAreas.get(trigger).get(owner).add(actionArea);
    }
}
